@@ -1,11 +1,11 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2019-2023, by Samuel Williams.
+# Copyright, 2019-2024, by Samuel Williams.
 # Copyright, 2019, by Soumya.
 # Copyright, 2021, by Aurora Nockert.
 
-require_relative 'error'
+require_relative "error"
 
 module Protocol
 	module WebSocket
@@ -34,6 +34,10 @@ module Protocol
 				@payload = payload
 			end
 			
+			def flag?(value)
+				@flags & value != 0
+			end
+			
 			def <=> other
 				to_ary <=> other.to_ary
 			end
@@ -46,6 +50,7 @@ module Protocol
 				@opcode & 0x8 != 0
 			end
 			
+			# @returns [Boolean] if the frame contains data.
 			def data?
 				false
 			end
@@ -86,6 +91,30 @@ module Protocol
 			attr_accessor :length
 			attr_accessor :payload
 			
+			if IO.const_defined?(:Buffer) && IO::Buffer.respond_to?(:for) && IO::Buffer.method_defined?(:xor!)
+				private def mask_xor(data, mask)
+					buffer = data.dup
+					mask_buffer = IO::Buffer.for(mask)
+					
+					IO::Buffer.for(buffer) do |buffer|
+						buffer.xor!(mask_buffer)
+					end
+					
+					return buffer
+				end
+			else
+				warn "IO::Buffer not available, falling back to slow implementation of mask_xor!"
+				private def mask_xor(data, mask)
+					result = String.new(encoding: Encoding::BINARY)
+						
+					for i in 0...data.bytesize do
+						result << (data.getbyte(i) ^ mask.getbyte(i % 4))
+					end
+					
+					return result
+				end
+			end
+			
 			def pack(data = "")
 				length = data.bytesize
 				
@@ -94,12 +123,7 @@ module Protocol
 				end
 				
 				if @mask
-					@payload = String.new(encoding: Encoding::BINARY)
-					
-					for i in 0...data.bytesize do
-						@payload << (data.getbyte(i) ^ mask.getbyte(i % 4))
-					end
-					
+					@payload = mask_xor(data, mask)
 					@length = length
 				else
 					@payload = data
@@ -111,13 +135,7 @@ module Protocol
 			
 			def unpack
 				if @mask and !@payload.empty?
-					data = String.new(encoding: Encoding::BINARY)
-					
-					for i in 0...@payload.bytesize do
-						data << (@payload.getbyte(i) ^ @mask.getbyte(i % 4))
-					end
-					
-					return data
+					return mask_xor(@payload, @mask)
 				else
 					return @payload
 				end
@@ -160,10 +178,10 @@ module Protocol
 				
 				if length == 126
 					buffer = stream.read(2) or raise EOFError, "Could not read length!"
-					length = buffer.unpack('n').first
+					length = buffer.unpack("n").first
 				elsif length == 127
 					buffer = stream.read(8) or raise EOFError, "Could not read length!"
-					length = buffer.unpack('Q>').first
+					length = buffer.unpack("Q>").first
 				end
 				
 				if length > maximum_frame_size
@@ -205,12 +223,12 @@ module Protocol
 				buffer << [
 					(@finished ? 0b1000_0000 : 0) | (@flags << 4) | @opcode,
 					(@mask ? 0b1000_0000 : 0) | short_length,
-				].pack('CC')
+				].pack("CC")
 				
 				if short_length == 126
-					buffer << [@length].pack('n')
+					buffer << [@length].pack("n")
 				elsif short_length == 127
-					buffer << [@length].pack('Q>')
+					buffer << [@length].pack("Q>")
 				end
 				
 				buffer << @mask if @mask
